package de.hawhamburg.reachability.sensor.adapter;

import java.text.SimpleDateFormat;

import org.apache.log4j.Logger;

import com.ibm.icu.util.Calendar;

import de.hawhamburg.reachability.reasoner.enumeration.ReasonerDimension;
import de.hawhamburg.reachability.sensor.Sensor;
import de.hawhamburg.reachability.sensor.SensorAdapter;
import de.hawhamburg.reachability.sensor.SensorAdapterFactory;
import de.hawhamburg.reachability.sensor.SensorAgent;
import de.hawhamburg.reachability.sensor.SensorData;
import de.hawhamburg.reachability.util.OutputUtil;

/**
 * Sensor to sense the current system time.<br><br>
 * 
 * This sensor provides the current time, each time the method {@link #getSensorOutput()} is called, as 
 * {@link SensorData} object.<br><br>
 * 
 * This is a good example for a sensor which doesn't base on the ActiveMQ communication, but on other data sources. 
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class TimeAdapter extends    Sensor
                         implements SensorAdapter  {
        
    /**
     * Constructs a new time sensor adapter which detects the current time.<br><br>
     * 
     * The sensors configurations will be loaded by the constructor of {@link Sensor} from the XML file in  
     * {@link de.hawhamburg.reachability.sensor.Settings#SENSOR_CONFIGURATION_PATH}
     * which is named like this class.
     * 
     * @param dimension The {@link ReasonerDimension} for which this sensor is configured.
     */
    public TimeAdapter(final ReasonerDimension dimension) {
        super(dimension);
    }
    
    /**
     * Provides the current time in format HHMMSS as {@link SensorData}.
     * 
     * @return          The current time in format HHMMSS as {@link SensorData}.
     */
    public final SensorData getSensorOutput() {
        double value = 0.0;
        
        String currTime = new SimpleDateFormat("HHmmss").format(Calendar.getInstance().getTime());
        value = Double.valueOf(currTime);
        OutputUtil.debug(currTime);
        
        return new SensorData(this.getClass().getSimpleName(), this.getDataDimension(), value);
    }

    /**
     * Main method which starts this {@link SensorAdapter} as {@link SensorAgent}.
     * 
     * @param args      The start arguments as array of {@link String}.
     */
    public static void main(final String[] args) {
        // Remove all existing loggers
        Logger.getRootLogger().removeAllAppenders();
        
        // Build this sensor adapter by using the SensorAdapterFactory.
        SensorAdapter sa = SensorAdapterFactory.getConnectedSensorAdapter(TimeAdapter.class.getSimpleName(),
                                                                          ReasonerDimension.time);
        
        // Start the sensor adapter as sensor agent, which supplied the required control functions.
        new SensorAgent(sa);
    }
}
