package de.hawhamburg.reachability.sensor;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.hawhamburg.lifecycle.agent.LifeCycleAgent;
import de.hawhamburg.lifecycle.agent.LifeCycleApp;
import de.hawhamburg.lifecycle.data.JsonInfoMessage;
import de.hawhamburg.lifecycle.data.ProjectStatus;
import de.hawhamburg.reachability.ProjectInfo;
import de.hawhamburg.reachability.reasoner.enumeration.ReasonerDimension;
import de.hawhamburg.reachability.util.OutputUtil;

/**
 * Class to manage the "Living Place Agent Life Cycle" for {@link SensorAdapter}s.<br><br>
 * 
 * This class handles the requests related to the "living place agent life cycle" specification, which will be supplied
 * on a global control topic via the ActiveMQ. It implements the {@link LifeCycleAgent} interface and will be 
 * instantiated with an {@link SensorAdapter}, which will be started immediately. Furthermore it will wrap the running 
 * {@link SensorAdapter} by a {@link LifeCycleApp}. Nevertheless, this class implements the control actions related to a
 * {@link SensorAdapter}, such as: "start", "sleep", "info" and "stop".
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR}
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class SensorAgent implements LifeCycleAgent {
    
    /**
     * The current status of the sensor agent.
     */
    private ProjectStatus myStatus;
    
    /**
     * Time to wait to check again if the worker has already been stopped, while {@link #stop()} is ending the worker 
     * thread, before sending success.  
     */
    private static final long SHUTDOWN_CHECKTIME    = 100;
    
    /**
     * The {@link SensorAdapter}, a thread which provides sensor data of a certain context dimension in an internal data
     * format.
     */
    private SensorAdapter worker                    = null;
    
    /**
     * The simple class name of the {@link #worker}.
     */
    private String workerName                       = null;
    
    /**
     * The context dimension as {@link ReasonerDimension} for which the {@link #worker} provides sensor data.
     */
    private ReasonerDimension workerDimension       = null;

    /**
     * Create a new {@link SensorAgent} which starts the supplied {@link SensorAdapter} as {@link LifeCycleApp} and
     * manages control messages for this {@link SensorAdapter}, such as "start", "stop", "info" and "sleep".
     * 
     * @param adapter   The {@link SensorAdapter} which shall be started and managed as {@link LifeCycleApp}.       
     */
    public SensorAgent(final SensorAdapter adapter) {
        this.worker             = adapter;
        this.workerName         = this.worker.getClass().getSimpleName();
        this.workerDimension    = this.worker.getDataDimension();
        
        OutputUtil.info("*******************************");
        OutputUtil.info("** New life cycle agent '" + this.getClass().getSimpleName() + "' started");
        OutputUtil.info("*******************************");
        OutputUtil.info("** - ID:            '" + this.getID() + "'");
        OutputUtil.info("** - Current state: '" + ProjectStatus.SLEEPING + "'");
        OutputUtil.info("*******************************");
        
        this.worker.start();
        
        LifeCycleApp lifecycle = new LifeCycleApp();
        lifecycle.setLifeCycleAgent(this);
        new Thread(lifecycle).start();
        
        this.myStatus = ProjectStatus.SLEEPING;
    }
    
    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#sleep()
     */
    public final void sleep() {
        OutputUtil.info("Getting Sleeping Order");
        this.myStatus = ProjectStatus.SLEEPING;
        this.worker.setSilent(true);
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#getInfoMessage()
     * 
     * @return          A {@link JsonInfoMessage}, which describes the current execution state of the managed 
     *                  {@link SensorAdapter}.
     */
    public final JsonInfoMessage getInfoMessage() {
        JsonInfoMessage msg = new JsonInfoMessage();

        msg.author  = ProjectInfo.REACHABILITY_AGENT_AUTHOR;
        msg.Id      = this.getID();
        msg.status  = this.myStatus.toString();
        msg.Version = ProjectInfo.REACHABILITY_AGENT_VERSION;
        try {
            msg.ip = InetAddress.getLocalHost().toString();
        } catch (UnknownHostException e) {
            Logger.getLogger(SensorAgent.class.getName()).log(Level.SEVERE, null, e);
            msg.ip = "unknown";
        }
        msg.misc    = ProjectInfo.REACHABILITY_AGENT_DESCRIPTION;
        
        OutputUtil.info("Sending INFO Message");
        
        return msg;
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#start()
     */
    public final void start() {
        OutputUtil.info("Getting Starting Order!");
        this.myStatus = ProjectStatus.RUNNING;
        
        this.worker.setSilent(false);
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#stop()
     */
    public final void stop() {
        OutputUtil.info("Getting Stopping Order!");
        this.myStatus = ProjectStatus.STOPPED;
        
        if (this.worker != null) {
            this.worker.shutDown();
            while (this.worker != null && !this.worker.stopped()) {
                try {
                    Thread.sleep(SHUTDOWN_CHECKTIME);
                } catch (InterruptedException e) {
                    System.err.println(this.getClass().getName() + ": Unable to sleep this thread. " + e.getMessage());
                }
            }
            this.worker = null;
        }

        OutputUtil.info("Worker has stopped!");
    }

    /**
     * @see de.hawhamburg.lifecycle.agent.LifeCycleAgent#getID()
     * 
     * @return          The ID of this service as {@link String} defined by {@link ProjectInfo#REACHABILITY_AGENT_NAME}.
     *                  This exact ID needs to be provided within a control message to take effect on this
     *                  {@link SensorAgent}.
     */
    public final String getID() {
        return ProjectInfo.REACHABILITY_AGENT_NAME + "_Sensor_"  + this.workerDimension + "_" + this.workerName;
    }
}
